# Copy! v1.3.1 plugin for Movable Type
# http://www.chipple.net/mt/CopyEntry/
# by Patrick Benny http://www.chipple.net/
# This code cannot be redistributed without permission from the author. 
#
# Version history
#   1.3.1 - 2011/8/21  - Fixed bug preventing access to plugin settings
#   1.3   - 2011/7/18  - Now copies tags
#   1.2.1 - 2010/12/13 - Now supports Movable Type 4.35
#                        Fixed bug related to categories
#   1.1   - 2008/6/29  - Now supports Movable Type 4.2
#   1.0   - 2008/5/28  - First public release
#                        Added copy link to <$MTEntryEditLink$>
#   0.9   - 2008/5/15  - Public preview release

package MT::Plugin::Copy;

use strict;
use base 'MT::Plugin';
use vars qw($VERSION);
$VERSION = '1.3.1';

my $plugin;
$plugin = MT::Plugin::Copy->new({
    name => '<MT_TRANS phrase="Copy!">',
    version => $VERSION,
    doc_link => "http://www.chipple.net/mt/Copy/",
    description => '<MT_TRANS phrase="Allows copying an entry, page or template to a new one.">',
    author_name => '<MT_TRANS phrase="Patrick Benny">',
    author_link => "http://www.chipple.net/",
    l10n_class => 'Copy::L10N',
    registry => {
        tags => {
            function => {
                EntryEditLink => \&_hdlr_entry_edit_link_copy,
            },
        },
    },
});
MT->add_plugin($plugin);

sub instance { $plugin }

## For Movable Type 4.0/4.1/4.2
MT->add_callback('MT::App::CMS::template_source.edit_entry', 9, $plugin, \&_template_entry_mt4);
MT->add_callback('MT::App::CMS::template_source.edit_template', 9, $plugin, \&_template_template_mt4);

# Keep a pointer to MT subs
my $mt41_edit = \&MT::App::CMS::edit_object;
my $mt42_edit = \&MT::CMS::Common::edit;

# Remap CMS methods edit and view to edit_copy
sub init_registry {
  my $plugin = shift;
  $plugin->registry({
    applications => {
      cms => {
        methods => {
          edit => \&edit_copy,
          view => \&edit_copy,
        },
      },
    },
  });
}


############
### MAIN ###
############

sub edit_copy {
  my $app  = shift;
  my $q    = $app->param;
  my $type = $q->param('_type');

  return $app->errtrans("Invalid request.")
    unless $type;
    
  # Ignore calls not from entry, page or template
  if ($type ne 'entry' && $type ne 'page' && $type ne 'template') {
    return;
  }

  if ( $q->param('copy') ) {
    if ($type eq 'entry' || $type eq 'page') {
      my $entry;
      if ( $entry = MT::Entry->load($q->param('copy')) ) {
        $q->param('allow_comments', $entry->allow_comments);
        $q->param('title',          $entry->title.$app->translate(' (copy)'));
        $q->param('excerpt',        $entry->excerpt);
        $q->param('text',           $entry->text);
        $q->param('text_more',      $entry->text_more);
        $q->param('convert_breaks', $entry->convert_breaks);
        $q->param('allow_pings',    $entry->allow_pings);
        $q->param('keywords',       $entry->keywords);
        $q->param('tags',           join(', ', $entry->tags));
        # Get list of all categories
        my $cats = join(',', map { $_->id } @{$entry->categories});
        # Remove primary category from list, and add it back to the start
        my $primcat = $entry->category->id if $entry->category;
        $cats =~ s/(,$primcat|^$primcat,|^$primcat\$)//;
        $cats = $primcat.','.$cats;
        $q->param('category_ids',   $cats);
      }
    }
    elsif ( ( $type eq 'template' ) ) {
      my $tmpl;
      if ( $tmpl = MT::Template->load($q->param('copy')) ) {
        $q->param('name',           $tmpl->name.$app->translate(' (copy)'));
        $q->param('type',           $tmpl->type);
        $q->param('text',           $tmpl->text);
        $q->param('rebuild_me',     $tmpl->rebuild_me);
        $q->param('build_dynamic',  $tmpl->build_dynamic);
        $q->param('identifier',     $tmpl->identifier);
      }
    }
  }

  return &$mt41_edit($app, @_) if $mt41_edit;
  return &$mt42_edit($app, @_) if $mt42_edit;
}

sub _hdlr_entry_edit_link_copy {
  my $ctx = shift;
  my $edit_link = $ctx->_hdlr_entry_edit_link($ctx,@_);
  my $copy_link = $edit_link;

  # Replace &id= to &copy=
  $copy_link =~ s|&(amp;)?id=|&amp;copy=|;

  # Replace "Edit" text to "Copy"
  my $edit_text = $plugin->translate('Edit');
  my $copy_text = $plugin->translate('copy_button_name');
  $copy_link =~ s|$edit_text|$copy_text|;

  return $edit_link.' '.$copy_link;
}


################
### TEMPLATE ###
################

sub get_copy_button {
  my $name = $plugin->translate('copy_button_name');
  return '
    <button type="submit" onclick="location.href=\'<mt:var name="script_url">?__mode=view&amp;_type=<mt:var name="object_type" lower_case="1">&amp;copy=<mt:var name="id">&amp;blog_id=<mt:var name="blog_id">\';return false;">'.$name.'</button>
';
}

sub _template_entry_mt4 {
    my ($cb, $app, $template) = @_;
    my $btn = get_copy_button();
    # For Movable Type 4.0/4.1
    $$template =~ s|(><__trans phrase="Delete"></button>)|$1$btn|;
}
sub _template_template_mt4 {
    my ($cb, $app, $template) = @_;
    my $btn = get_copy_button();
    # For Movable Type 4.0/4.1
    $$template =~ s|(    </mt:unless>[\r\n]+    <div id="autosave-notification-)|$btn$1|;
}

1;